@@ -1,6 +1,12 @@
 test_that("Object retrieval function work", {
   sc <- use_test_spark_connect()
   tbl_mtcars <- use_test_table_mtcars()
+
+  expect_s3_class(
+    spark_ide_preview.pyspark_connection(sc, table = "mtcars", rowLimit = 10),
+    "data.frame"
+  )
+
   expect_s3_class(spark_ide_objects(sc), "data.frame")
 
   expect_snapshot(
@@ -20,4 +26,10 @@ test_that("Object retrieval function work", {
     catalog_python(sc, catalog = "spark_catalog", schema = "default"),
     "data.frame"
   )
+
+  expect_s3_class(
+    rs_get_databases(sc, limit = 10,  catalog = "spark_catalog"),
+    "data.frame"
+  )
+
 })
